@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 from gmusicapi._version import __version__
@@ -7,11 +6,11 @@ __copyright__ = 'Copyright 2013 Simon Weber'
 __license__ = 'BSD 3-Clause'
 __title__ = 'gmusicapi'
 
-from gmusicapi.clients import Webclient, Musicmanager
+from gmusicapi.clients import Webclient, Musicmanager, Mobileclient
 from gmusicapi.exceptions import CallFailure
 
 # appease flake8: the imports are purposeful
-(__version__, Webclient, Musicmanager, CallFailure)
+(__version__, Webclient, Musicmanager, Mobileclient, CallFailure)
 
 
 class Api(object):
