@@ -205,6 +205,8 @@ class Call(object):
 
         req_kwargs = cls.build_request(*args, **kwargs)
 
+        log.debug(req_kwargs)
+
         response = session.send(req_kwargs, cls.required_auth)
         #TODO trim the logged response if it's huge?
 
@@ -246,8 +248,20 @@ class Call(object):
             cls.check_success(response, parsed_response)
             if validate:
                 cls.validate(response, parsed_response)
-        except CallFailure:
-            raise
+        except CallFailure as e:
+            if not cls.gets_logged:
+                raise
+
+            # otherwise, reraise a new exception with our req/res context
+            trace = sys.exc_info()[2]
+            err_msg = ("{e_message}\n"
+                       "(requests kwargs: {req_kwargs!r})\n"
+                       "(response was: {content!r})").format(
+                           e_message=e.message,
+                           req_kwargs=safe_req_kwargs,
+                           content=response.content)
+            raise CallFailure(err_msg, e.callname), None, trace
+
         except ValidationException as e:
             #TODO shouldn't be using formatting
             err_msg = "the response format for %s was not recognized." % call_name
